      subroutine root

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  root     Updated     2004 July 27 15:20.
c               root     Originated  1997 July 31 16:50.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find real roots, extrema and inversion points of a polynomial
c               equation with real coefficients A0, A1, A2, A3, A4, ...:
c               A0 + A1 * x + A2 * x**2 + A3 * x**3 + A4 * x**4 + ... = 0.
c               using Newtonian iteration, given a minimum, increment and
c               maximum value of the initial guess for x, and a maximum number
c               of iterations.  Convergence is determined by tol.
c               Also, to find the coefficients of a polynomial equation of
c               order N, given N real roots.
c
c     Logic:    Entry points:  root.
c root          Called by:  helpall, main.
c
c root          Calls:  aptnewt, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:  2001 January 22 15:00.  Added tests for failure to find absolute
c               limits on roots, extrema and inflection points.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      dimension a(0:64)               ! Coefficients of polynomial equation.
      dimension aa(0:64)              ! Modified values of a.
      dimension aerr(0:64)            ! Error in coefficients of polynomial.
      dimension b(0:64)               ! Coefficients of 1st derivative equation.
      dimension bb(0:64)              ! Modified values of b.
      dimension c(0:64)               ! Coefficients of 2nd derivative equation.
      dimension cc(0:64)              ! Modified values of c.
      dimension xpoly(1000)           ! Roots, extrema, inflection point x.
      dimension   apoly(1000)         ! Roots, extrema, inflection point type.
      character*8 apoly               ! Roots, extrema, inflection point type.
      character*8 apolyx              ! Roots, extrema, inflection point type.

c=======================================================================********

c.... Input line:  root [help]

 8110 format ('  Command "root":  acceptable input lines include:' /
     &  '  "root [help]"           Display this message.' //
     &  '  "root XMIN DX XMAX NITMAX A(0) A(1) A(2) A(3) ... A(N)"' /
     &  '     Search for real roots, extrema and inversion points of',
     &  ' the polynomial' /
     &  '     equation P(x) = sum(n=0,N) {A(n) * x**n} = 0,',
     &  ' by Newtonian iteration,' /
     &  '     with initial guesses of x from XMIN to XMAX,',
     &  ' with increment DX,' /
     &  '     and no more than NITMAX iterations.' //
     &  '  "root = X(1) X(2) X(3) ... X(N)"' /
     &  '     Find the coefficients a(n) (n=0,N) for the polynomial' /
     &  '     equation P(x) = sum(n=0,N) {a(n) * x**n} = 0,' /
     &  '     with the roots X(1), X(2), X(3), ..., X(N).' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  root = X(1) X(2) X(3) ... X(N)

      if (aword(2) .eq. '=') then     ! Specify roots, find coefficients.

      nerr = 0

 8320 format ('  root ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not enough words on "root =" line:  ', i3)

      if (nword .lt. 3) then
        write (   6, 8320) ainput, ninput, nword
        write (iout, 8320) ainput, ninput, nword
        go to 999
      endif

c.... Find the input data.

      call varfind (3, nword)

      do 340 nw = 3, nword
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)
          go to 999
        endif
  340 continue

c.... Find the roots of the polynomial equation.

      norda = nword - 2

      do 345 n = 1, norda
        xpoly(n) = fword(n+2)
  345 continue

c.... Initialize all coefficients to zero.

      do 350 n = 0, 64
        a(n)    = 0.0
        aerr(n) = 0.0
  350 continue

c.... Find the coefficients for the first root.

      a(1)    =  1.0
      aerr(1) =  0.0
      a(0)    = -xpoly(1)
      aerr(0) =  tol * abs (xpoly(1))

c.... Find the coefficients based on all remaining roots.

      do 370 m = 2, norda
        a(m)    = 1.0
        aerr(m) = 0.0
        xm      = xpoly(m)
        do 360 n = m - 1, 1, -1
        aerr(n) = aerr(n-1) + abs (xm) * (tol * abs (a(n)) + aerr(n))
        a(n)    = a(n-1) - xm * a(n)
  360   continue
        aerr(0) =  abs (xm) * (tol * abs (a(0)) + aerr(0))
        a(0)    = -xm * a(0)
  370 continue
cbugc***DEBUG begins.
cbug 9977 format ('n=',i2,'  a, aerr=',1p2e20.12)
cbug      write (iout, 9977) (n, a(n), aerr(n), n = 1, norda)
cbugc***DEBUG ends.

      do 375 n = 0, norda
        if (abs (a(n)) .le. aerr(n)) a(n) = 0.0
  375 continue

c.... Display all coefficients.

 8380 format ('  set   a',i1,'      = ',1pe20.12,'  ! Cut and paste.')
 8390 format ('  set   a',i2,'     = ',1pe20.12,'  ! Cut and paste.')

      do 380 n = 0, norda
        if (n .lt. 10) then
          write (   6, 8380) n, a(n)
          write (iout, 8380) n, a(n)
        else
          write (   6, 8390) n, a(n)
          write (iout, 8390) n, a(n)
        endif
  380 continue

      go to 999

      endif                           ! Specify roots, find coefficients.

c=======================================================================********

c.... Input line:  root XMIN DX XMAX NITMAX A(0) A(1) A(2) A(3) ... A(N)

      nerr  = 0

 8120 format ('  root ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not enough words on "root" line:  ', i3)

      if (nword .lt. 7) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the input data.

      call varfind (2, nword)

 8130 format ('  root ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      do 140 nw = 2, nword
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)
          nerr = 1
        endif
  140 continue

c.... Find the initial guess and the search range.

      xmin  = fword(2)
      dx    = fword(3)
      xmax  = fword(4)

      if (dx .eq. 0.0) dx = xmax - xmin

 8140 format ('  root ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  XMIN => XMAX or DX < 0:  ' /
     &  '  XMIN,DX,XMAX=',1p3e20.12 )

      if ((xmin .gt. xmax) .or.
     &    (dx .lt. 0.0))   then
          write (   6, 8140) ainput, ninput, xmin, dx, xmax
          write (iout, 8140) ainput, ninput, xmin, dx, xmax
          nerr = 1
      endif

c.... Find the maximum number of iterations.

 8150 format ('  root ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  NITMAX not a non-negative integer:  ',a)

      if ((mtype(5) .ne. 2)  .or.
     &    (iword(5) .lt. 0)) then
          write (   6, 8150) ainput, ninput, aword(5)
          write (iout, 8150) ainput, ninput, aword(5)
          nerr = 1
      endif

      nitmax = iword(5)
      if (nitmax .eq. 0) nitmax = 100

c.... Ignore trailing zeros on input line.

  115 if (fword(nword) .eq. 0.0) then
        nword = nword - 1
        go to 115
      endif

      if (nword .lt. 7) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        nerr = 1
      endif

      if (nerr .ne. 0) go to 999

c.... Initialize.

      ntota    = 0                    ! Total # of roots.
      ntotb    = 0                    ! Total # of extrema.
      ntotc    = 0                    ! Total # of inflection points.
      ntott    = 0                    ! Total # of roots, extrema, infl. pts.
      ntotmax  = 1000                 ! Maximum size of arrays xpoly, apoly.
      xlima    = 1.e99                ! Limiting absolute value of x (root).
      xlimb    = 1.e99                ! Limiting absolute value of x (extremum).
      xlimc    = 1.e99                ! Limiting absolute value of x (inflectn).

      do 145 n = 0, 64
        a(n) = 0.0
        b(n) = 0.0
        c(n) = 0.0
  145 continue

      do 148 n = 1, 1000
        xpoly(n) = -1.e99
        apoly(n) = 'unknown'
  148 continue

c.... Find the order of the polynomial equation.

      norda = nword - 6

c.... Find the coefficients of the polynomial equation.

      do 150 n = 0, norda
        a(n) = fword(n+6)
  150 continue

c.... Check for leading zero coefficients of the polynomial equation.

 8270 format ('  x (root)       =',1pe20.12)

      if (a(0) .eq. 0.0) then         ! One root is zero.
        ntott = ntott + 1
        if (ntott .gt. ntotmax) ntott = ntotmax
        xpoly(ntott) = 0.0
        apoly(ntott) = 'root'
        xroota = 0.0
        write (   6, 8270) xroota
        write (iout, 8270) xroota
      endif                           ! Tested for leading zero coefficient.

c.... Find the coefficients of the 1st derivative equation.

      nordb = norda - 1
      if (nordb .ge. 0) then
        do 156 n = 0, nordb
          b(n) = (n + 1) * a(n+1)
  156   continue
      endif

c.... Check for leading zero coefficient of the 1st derivative equation.

cbug 8275 format ('  x (min or max) =',1pe20.12)

      if ((nordb .ge. 1) .and. (b(0) .eq. 0.0)) then  ! Extremum at x = 0.
        ntott = ntott + 1
        if (ntott .gt. ntotmax) ntott = ntotmax
        xpoly(ntott) = 0.0
        apoly(ntott) = 'extremum'
        xrootb = 0.0
cbug        write (   6, 8275) xrootb
cbug        write (iout, 8275) xrootb
      endif                           ! Tested for leading zero coefficient.

c.... Find the coefficients of the 2nd derivative equation.

      nordc = nordb - 1
      if (nordc .ge. 0) then          ! Test order.
        do 162 n = 0, nordc
          c(n) = (n + 1) * b(n+1)
  162   continue
      endif                           ! Order is 0 or more.
cbugc***DEBUG begins.
cbug 9162 format ('  n, a, b, c =',i3,1p3e20.12)
cbug      write (   6, 9162) (n, a(n), b(n), c(n), n = 0, norda)
cbug      write (iout, 9162) (n, a(n), b(n), c(n), n = 0, norda)
cbugc***DEBUG ends.

c.... Check for leading zero coefficient of the 2nd derivative equation.

cbug 8280 format ('  x (inflection) =',1pe20.12)

      if ((nordc .ge. 1) .and. (c(0) .eq. 0.0)) then  ! Inflection at x = 0.
        ntott = ntott + 1
        if (ntott .gt. ntotmax) ntott = ntotmax
        xpoly(ntott) = 0.0
        apoly(ntott) = 'inflectn'
        xrootc = 0.0
cbug        write (   6, 8280) xrootc
cbug        write (iout, 8280) xrootc
      endif                         ! Tested for leading zero coefficient.

c.... Find the limit on the positive value of the roots.

      do 152 n = 0, norda - 1
        aa(n) = abs (a(n))
  152 continue
      aa(norda) = -abs (a(norda))

      xmint = tol
      if (xmint .eq. 0.0) xmint = 1.e-11
      xtry  = max (abs (xmin), abs (xmax))

      call aptnewt (aa, norda, xtry, xmint, 1.e20, 1000, tol,
     &              xlima, nerra)

 8282 format ('  root ERROR.  aptnewt failed.  Reduce limits.')

      if (nerra .ne. 0) then
        write (   6, 8282)
        write (iout, 8282)
        xlima = abs (xmax)
      endif

c.... Find the limit on the absolute value of the extrema.

      do 153 n = 0, nordb - 1
        bb(n) = abs (b(n))
  153 continue
      bb(nordb) = -abs (b(nordb))

      call aptnewt (bb, nordb, xtry, xmint, 1.e20, 1000, tol,
     &              xlimb, nerra)

      if (nerra .ne. 0) then
        write (   6, 8282)
        write (iout, 8282)
        xlimb = abs (xmax)
      endif

c.... Find the limit on the absolute value of the inflection points.

      do 154 n = 0, nordc - 1
        cc(n) = abs (c(n))
  154 continue
      cc(nordc) = -abs (c(nordc))

      call aptnewt (cc, nordc, xtry, xmint, 1.e20, 1000, tol,
     &              xlimc, nerra)

      if (nerra .ne. 0) then
        write (   6, 8282)
        write (iout, 8282)
        xlimc = abs (xmax)
      endif

c.... Display limiting values.

 8265 format (/ ' |x (root)    | <=',1pe20.12)
 8266 format (' |x (extremum)| <=',1pe20.12)
 8267 format (' |x (inflectn)| <=',1pe20.12)

      write (   6, 8265) xlima
      write (iout, 8265) xlima
      write (   6, 8266) xlimb
      write (iout, 8266) xlimb
      write (   6, 8267) xlimc
      write (iout, 8267) xlimc

c.... Change limits, if outside maximum possible range.

 8154 format (/ '  Changed XMIN to ',1pe20.12,' from ',1pe20.12)
 8155 format (/ '  Changed XMAX to ',1pe20.12,' from ',1pe20.12)
 8156 format (/ '  Changed DX   to ',1pe20.12,' from ',1pe20.12)

      xrange  = xmax - xmin
      xmaxall = max (xlima, xlimb, xlimc)
      xminall = -xmaxall

      klim = 0
      if (xmin .lt. xminall) then
        klim = 1
        write (   6, 8154) xminall, xmin
        write (iout, 8154) xminall, xmin
        xmin = xminall
      endif
      if (xmin .gt.  xmaxall) then
        klim = 1
        write (   6, 8154) xminall, xmax
        write (iout, 8154) xminall, xmax
        xmin =  xmaxall
      endif
      if (xmax .lt. xminall) then
        klim = 1
        write (   6, 8155) xmaxall, xmin
        write (iout, 8155) xmaxall, xmin
        xmax = xminall
      endif
      if (xmax .gt.  xmaxall) then
        klim = 1
        write (   6, 8155) xmaxall, xmax
        write (iout, 8155) xmaxall, xmax
        xmax =  xmaxall
      endif

      if ((klim .eq. 1) .and. (xrange .gt. 0.0)) then
        dxold = dx
        dx    = dx * (xmax - xmin) / xrange
        write (   6, 8156) dx, dxold
        write (iout, 8156) dx, dxold
      endif

c.... Find the number of trial values of x.

      if (dx .eq. 0.0) then
        ntry = 1
      else
        ntry  = (xmax - xmin) / dx + 2.0
      endif

c.... Initialize for the loop over trial values of x.

      xtry   = xmin                   ! Trial value of x.
      xerr   = 0.0                    ! Error in calculating xtry.
      polda  = 0.0                    ! Value of polynomial.
      poldb  = 0.0                    ! Value of 1st derivative of polynomial.
      poldc  = 0.0                    ! Value of 2nd derivative of polynomial.
      nmod   = 1 + (ntry - 1) / 32    ! Index interval between edits.

c.... Loop over the trial values of x.

 8210 format (/ '  Trial',15x,'x',19x,'P(x)',9x,'P''(x)',8x,'P''''(x)')

cbugc***DEBUG begins.
cbug      write (   6, 8210)
cbug      write (iout, 8210)
cbugc***DEBUG ends.

      do 260 nt = 1, ntry             ! Loop over trial values of x.

c....   Find the value of the polynomial equation, its 1st derivative
c....     and 2nd derivative at xtry.

        ptrya = a(0)                  ! Value of polynomial.
        ptryb = b(0)                  ! Value of 1st derivative of polynomial.
        ptryc = c(0)                  ! Value of 2nd derivative of polynomial.
        perra = 0.0                   ! Error in polynomial value.
        perrb = 0.0                   ! Error in 1st derivative value.
        perrc = 0.0                   ! Error in 2nd derivative value.
        xperr = 0.0                   ! Error in x**n, n = 0, norda.
        xpow  = 1.0                   ! Value of x**n, n = 0, norda.

        do 220 n = 1, norda           ! Loop over remaining coefficients.
          xperr = abs (xtry) * (xperr + tol * abs (xpow))
          xpow  = xpow * xtry
          ptrya = ptrya + a(n) * xpow
          ptryb = ptryb + b(n) * xpow
          ptryc = ptryc + c(n) * xpow
          perra = perra + abs (a(n)) * (xperr + tol * abs (xpow))
          perrb = perrb + abs (b(n)) * (xperr + tol * abs (xpow))
          perrc = perrc + abs (c(n)) * (xperr + tol * abs (xpow))
  220   continue                      ! End of loop over coefficients.

        if (abs (ptrya) .le. perra) ptrya = 0.0
        if (abs (ptryb) .le. perrb) ptryb = 0.0
        if (abs (ptryc) .le. perrc) ptryc = 0.0

c....   Display argument and function values up to 32 times.

cbug 8240   format ('  x (trial)      =',1pe20.12,1p3e13.5)
 8245   format ('  x (trial)      =',1pe20.12 /
     &          '  P(x)           =',1pe20.12 /
     &          '  P''(x)          =',1pe20.12 /
     &          '  P''''(x)         =',1pe20.12 )

        kdisp = 0
        if ((mod (nt, nmod) .eq. 0) .or.
     &      (xtry .eq. xmin)       .or.
     &      (xtry .eq. xmax))      then
          kdisp = 1
          if (xmin .eq. xmax) then
            write (   6, 8245) xtry, ptrya, ptryb, ptryc
            write (iout, 8245) xtry, ptrya, ptryb, ptryc
cbug          else
cbug            write (   6, 8240) xtry, ptrya, ptryb, ptryc
cbug            write (iout, 8240) xtry, ptrya, ptryb, ptryc
          endif
        endif

c....   Find the search limits for this trial x.

        xmint = xtry - dx
        if (xmint .lt. xmin) xmint = xmin
        xmaxt = xtry + dx
        if (xmaxt .gt. xmax) xmaxt = xmax

c....   Try to find a root of the polynomial equation if the polynomial
c....     or the slope changes sign in the preceding interval,

cbug 8274 format ('  x (root)       =',1pe20.12)

        if (ptrya .eq. 0.0) then      ! Root at xtry.

          ntott = ntott + 1
          if (ntott .gt. ntotmax) ntott = ntotmax
          xpoly(ntott) = xtry
          apoly(ntott) = 'root'

cbug          write (   6, 8274) xtry
cbug          write (iout, 8274) xtry

        elseif (ptrya * polda .lt. 0.0) then  ! Polynomial changed sign.

          if (kdisp .eq. 0) then
            kdisp = 1
cbug            write (   6, 8240) xtry, ptrya, ptryb, ptryc
cbug            write (iout, 8240) xtry, ptrya, ptryb, ptryc
          endif

          call aptnewt (a, norda, xtry, xmint, xmaxt, nitmax, tol,
     &                  xroota, nerra)

          if (nerra .eq. 0) then

            ntott = ntott + 1
            if (ntott .gt. ntotmax) ntott = ntotmax
            xpoly(ntott) = xroota
            apoly(ntott) = 'root'

cbug            write (   6, 8274) xroota
cbug            write (iout, 8274) xroota
          endif

          if (xroota .gt. xtry) then  ! Try an interpolated x value.

            xint = (xmint * ptrya - xtry * polda) / (ptrya -polda)
            if (xint .lt. xmint) xint = xmint
            if (xint .gt. xtry)  xint = xtry

            call aptnewt (a, norda, xint, xmint, xtry, nitmax, tol,
     &                    xroota, nerra)

            if (nerra .eq. 0) then

              ntott = ntott + 1
              if (ntott .gt. ntotmax) ntott = ntotmax
              xpoly(ntott) = xroota
              apoly(ntott) = 'root'

cbug              write (   6, 8274) xroota
cbug              write (iout, 8274) xroota
            endif

          endif                       ! Tried an interpolated x value.

        elseif (ptryb * poldb .le. 0.0) then  ! Slope zero or changed sign.

          if (kdisp .eq. 0) then
            kdisp = 1
cbug            write (   6, 8240) xtry, ptrya, ptryb, ptryc
cbug            write (iout, 8240) xtry, ptrya, ptryb, ptryc
          endif

          call aptnewt (a, norda, xtry, xmint, xmaxt, nitmax, tol,
     &                  xroota, nerra)

          if (nerra .eq. 0) then

            ntott = ntott + 1
            if (ntott .gt. ntotmax) ntott = ntotmax
            xpoly(ntott) = xroota
            apoly(ntott) = 'root'

cbug            write (   6, 8274) xroota
cbug            write (iout, 8274) xroota
          endif

          if (xroota .gt. xtry) then  ! Try an interpolated x value.

            pdiffb = ptryb - poldb
            perrb  = tol * (abs (ptryb) + abs (poldb))
            if (abs (pdiffb) .le. perrb) pdiffb = 0.0

            if (pdiffb .ne. 0.0) then
              xint = (xmint * ptryb - xtry * poldb) / pdiffb
            else
              xint = 0.5 * (xmint + xtry)
            endif
            if (xint .lt. xmint) xint = xmint
            if (xint .gt. xtry)  xint = xtry

            call aptnewt (a, norda, xint, xmint, xtry, nitmax, tol,
     &                    xroota, nerra)

            if (nerra .eq. 0) then

              ntott = ntott + 1
              if (ntott .gt. ntotmax) ntott = ntotmax
              xpoly(ntott) = xroota
              apoly(ntott) = 'root'

cbug              write (   6, 8274) xroota
cbug              write (iout, 8274) xroota
            endif

          endif                       ! Tried an interpolated x value.

        endif                         ! Tried to find a root.

c....   Try to find an extremum, if the slope changes sign in the
c....     preceding interval.

        if (ptryb .eq. 0.0) then

          ntott = ntott + 1
          if (ntott .gt. ntotmax) ntott = ntotmax
          xpoly(ntott) = xtry
          apoly(ntott) = 'extremum'


cbug          write (   6, 8275) xtry
cbug          write (iout, 8275) xtry

        elseif (ptryb * poldb .lt. 0.0) then

          call aptnewt (b, nordb, xtry, xmint, xmaxt, nitmax, tol,
     &                  xrootb, nerrb)

          if (nerrb .eq. 0) then

            ntott = ntott + 1
            if (ntott .gt. ntotmax) ntott = ntotmax
            xpoly(ntott) = xrootb
            apoly(ntott) = 'extremum'

cbug            write (   6, 8275) xrootb
cbug            write (iout, 8275) xrootb
          endif

          if (xrootb .gt. xtry) then  ! Try an interpolated x value.

            xint = (xmint * ptryb - xtry * poldb) / (ptryb -poldb)
            if (xint .lt. xmint) xint = xmint
            if (xint .gt. xtry)  xint = xtry

            call aptnewt (b, nordb, xint, xmint, xtry, nitmax, tol,
     &                    xrootb, nerrb)

            if (nerrb .eq. 0) then

              ntott = ntott + 1
              if (ntott .gt. ntotmax) ntott = ntotmax
              xpoly(ntott) = xrootb
              apoly(ntott) = 'extremum'

cbug              write (   6, 8275) xrootb
cbug              write (iout, 8275) xrootb
            endif

          endif                       ! Tried an interpolated x value.

        endif                         ! Tried to find an extremum.

c....   Try to find an inflection point, if the 2nd derivative changes
c....     sign in the preceding interval.

        if ((nordc .ge. 1) .and. (ptryc .eq. 0.0)) then

          ntott = ntott + 1
          if (ntott .gt. ntotmax) ntott = ntotmax
          xpoly(ntott) = xtry
          apoly(ntott) = 'inflectn'

cbug          write (   6, 8280) xtry
cbug          write (iout, 8280) xtry

        elseif (ptryc * poldc .lt. 0.0) then

          if (kdisp .eq. 0) then
            kdisp = 1
cbug            write (   6, 8240) xtry, ptrya, ptryb, ptryc
cbug            write (iout, 8240) xtry, ptrya, ptryb, ptryc
          endif

          call aptnewt (c, nordc, xtry, xmint, xmaxt, nitmax, tol,
     &                  xrootc, nerrc)

          if (nerrc .eq. 0) then

            ntott = ntott + 1
            if (ntott .gt. ntotmax) ntott = ntotmax
            xpoly(ntott) = xrootc
            apoly(ntott) = 'inflectn'

cbug            write (   6, 8280) xrootc
cbug            write (iout, 8280) xrootc
          endif

          if (xrootc .gt. xtry) then  ! Try an interpolated x value.

            xint = (xmint * ptryc - xtry * poldc) / (ptryc -poldc)
            if (xint .lt. xmint) xint = xmint
            if (xint .gt. xtry)  xint = xtry

            call aptnewt (c, nordc, xint, xmint, xtry, nitmax, tol,
     &                    xrootc, nerrc)

            if (nerrc .eq. 0) then

              ntott = ntott + 1
              if (ntott .gt. ntotmax) ntott = ntotmax
              xpoly(ntott) = xrootc
              apoly(ntott) = 'inflectn'

cbug              write (   6, 8280) xrootc
cbug              write (iout, 8280) xrootc
            endif

          endif                       ! Tried an interpolated x value.

        endif                         ! Tried to find an inflection point.

c....   Save old values of polynomial, 1st and 2nd derivatives.

        polda = ptrya
        poldb = ptryb
        poldc = ptryc

c....   See if maximum x value has been reached.

        if (xtry .eq. xmax) go to 410

c....   Find next x value.

        xtry = xtry + dx
        xerr = xerr + tol * (abs (xtry) + abs (dx))
        if (abs (xtry) .le. xerr) xtry = 0.0
        if (abs (xtry - xmax) .le. xerr) xtry = xmax
        if (xtry .gt. xmax) xtry = xmax

  260 continue                        ! End of loop over trial values of x.

  410 if (ntott .eq. 0) go to 710

c.... Put all x values in ascending order.

      do 440 n = 1, ntott - 1
        do 430 nn = n + 1, ntott
          if (xpoly(nn) .lt. xpoly(n)) then
            xpolyx    = xpoly(n)
            apolyx    = apoly(n)
            xpoly(n)  = xpoly(nn)
            apoly(n)  = apoly(nn)
            xpoly(nn) = xpolyx
            apoly(nn) = apolyx
          endif
  430   continue
  440 continue

c.... Eliminate duplicate values of an event.

      nn = 1
      do 450 n = 2, ntott
        xdiff  = xpoly(n) - xpoly(n-1)
        xdiffe = tol * (abs (xpoly(n)) + abs (xpoly(n-1)))
        if (abs (xdiff) .le. xdiffe) xdiff = 0.0
        if (xdiff .ne. 0.0) then
          nn = nn + 1
          xpoly(nn) = xpoly(n)
          apoly(nn) = apoly(n)
        endif
  450 continue
      ntott = nn

 8450 format (/ '  Event',15x,'x',19x,'P(x)',9x,'P''(x)',8x,'P''''(x)')

      write (   6, 8450)
      write (iout, 8450)

c.... Find values of polynomial, 1st derivative, 2nd derivative at x values.

      do 480 n = 1, ntott             ! Loop over trial values of x.

        polya = a(0)                  ! Value of polynomial.
        polyb = b(0)                  ! Value of 1st derivative of polynomial.
        polyc = c(0)                  ! Value of 2nd derivative of polynomial.
        perra = 0.0                   ! Error in polynomial value.
        perrb = 0.0                   ! Error in 1st derivative value.
        perrc = 0.0                   ! Error in 2nd derivative value.
        xperr = 0.0                   ! Error in x**n, n = 0, norda.
        xpow  = 1.0                   ! Value of x**n, n = 0, norda.

        do 460 nn = 1, norda          ! Loop over remaining coefficients.
          xperr = abs (xpoly(n)) * (xperr + tol * abs (xpow))
          xpow  = xpow * xpoly(n)
          polya = polya + a(nn) * xpow
          polyb = polyb + b(nn) * xpow
          polyc = polyc + c(nn) * xpow
          perra = perra + abs (a(nn)) * (xperr + tol * abs (xpow))
          perrb = perrb + abs (b(nn)) * (xperr + tol * abs (xpow))
          perrc = perrc + abs (c(nn)) * (xperr + tol * abs (xpow))
  460   continue                      ! End of loop over coefficients.

        if (abs (polya) .le. perra) polya = 0.0
        if (abs (polyb) .le. perrb) polyb = 0.0
        if (abs (polyc) .le. perrc) polyc = 0.0

        apoly(n) = 'unknown'

        if ((nordc .ge. 1) .and. (polyc .eq. 0.0)) then
          ntotc    = ntotc + 1
          apoly(n) = 'inflectn'
        endif

        if (polyb .eq. 0.0) then      ! Extremum.
          ntotb = ntotb + 1
          if (polyc .lt. 0.0) then
            apoly(n) = 'maximum'
          elseif (polyc .gt. 0.0) then
            apoly(n) = 'minimum'
          elseif (polyc .eq. 0.0) then
            apoly(n) = 'min/max'
          endif
        endif

        if (polya .eq. 0.0) then      ! Root.
          ntota    = ntota + 1
          if (apoly(n) .eq. 'maximum') then
            apoly(n) = 'dbl max'
          elseif (apoly(n) .eq. 'minimum') then
            apoly(n) = 'dbl min'
          elseif (apoly(n) .eq. 'min/max') then
            apoly(n) = 'triple'
          else
            apoly(n) = 'single'
          endif
        endif

 8410   format ('  x (',a,')   =',1pe20.12,1p3e13.5)

        write (   6, 8410) apoly(n), xpoly(n), polya, polyb, polyc
        write (iout, 8410) apoly(n), xpoly(n), polya, polyb, polyc

 8420   format ('    1 / Root     =',1pe20.12 )

        if (polya .eq. 0.0) then
          if (xpoly(n) .eq. 0.0) then
            xreal = -1.e99
          else
            xreal = 1.0 / xpoly(n)
          endif
          write (   6, 8420) xreal
          write (iout, 8420) xreal
        endif

  480 continue

 8480 format (/ 'Total number of roots =',i3,', extrema =',i3,
     &  ', inflection points =',i3,'.' /
     &  'Total of all of these =',i3,'.')

  710 write (   6, 8480) ntota, ntotb, ntotc, ntott
      write (iout, 8480) ntota, ntotb, ntotc, ntott

 8490 format (/ 'Total unknown   roots =',i3,', extrema =',i3,
     &  ', inflection points =',i3,'.' )

      nunka = norda - ntota
      nunkb = nordb - ntotb
      nunkc = nordc - ntotc

      write (   6, 8490) nunka, nunkb, nunkc
      write (iout, 8490) nunka, nunkb, nunkc

 8520 format ('  root ERROR.  Ran out of storage space for roots.')

      if (ntott .eq. ntotmax) then
        write (   6, 8520)
        write (iout, 8520)
      endif

  999 return

c.... End of subroutine root.         (+1 line)
      end
